"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createNode = createNode;
exports.addNode = addNode;
exports.copyNode = copyNode;
exports.removeNode = removeNode;
exports.moveNode = moveNode;
exports.moveNodeInChildren = moveNodeInChildren;
exports.updateNodeDataProp = updateNodeDataProp;
exports.removeNodeDataProp = removeNodeDataProp;
exports.updateNodeProp = updateNodeProp;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _fastLoops = require("fast-loops");

var _getDefaultProps = _interopRequireDefault(require("./getDefaultProps"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function createNode(type, lorrenTypes = {}, props = {}, data = {}, children = []) {
  const defaultProps = (0, _getDefaultProps.default)(lorrenTypes);
  return {
    type,
    data,
    props: _objectSpread(_objectSpread({
      __displayName: type || props.__displayName
    }, defaultProps), props),
    children
  };
}

function addNode(nodes, node, index, parent = '__root') {
  return _objectSpread(_objectSpread({}, nodes), {}, {
    [index]: node,
    [parent]: _objectSpread(_objectSpread({}, nodes[parent]), {}, {
      children: [...nodes[parent].children, index]
    })
  });
}

function recursiveAddNode(nodes, node, index, parent) {
  const children = node.children.reduce((children, childId) => {
    const newIndex = index + 1;
    index = recursiveAddNode(nodes, nodes[id], newIndex, node.id);
    return [...children, newIndex];
  }, []);
  nodes = addNode(nodes, _objectSpread(_objectSpread({}, node), {}, {
    children
  }), index, parent);
  return index;
}

function recurAdd(nodes, node, index, parent) {
  const parentIndex = index;
  const children = node.children;
  return children.reduce(([newNodes, newIndex], childId) => {
    return recurAdd(newNodes, _objectSpread({}, newNodes[childId]), newIndex + 1, parentIndex);
  }, [addNode(nodes, _objectSpread(_objectSpread({}, node), {}, {
    children: []
  }), index, parent), index]);
}

function copyNode(nodes, node, index, parent = '__root', setId) {
  const [newNodes, newIndex] = recurAdd(nodes, node, index + 1, parent);
  setId(newIndex);
  return newNodes;
}

function recursiveRemoveNodes(nodes, nodeId) {
  const cleanedNodes = nodes[nodeId].children.reduce((newNodes, id) => recursiveRemoveNodes(newNodes, id), nodes);
  return Object.keys(cleanedNodes).filter(id => id !== nodeId).reduce((newNodes, id) => {
    newNodes[id] = _objectSpread(_objectSpread({}, cleanedNodes[id]), {}, {
      children: cleanedNodes[id].children.filter(childId => childId !== nodeId)
    });
    return newNodes;
  }, {});
}

function removeNode(nodes, nodeId) {
  return recursiveRemoveNodes(nodes, nodeId);
}

function moveNode(nodes, nodeIndex, oldParent, newParent) {
  if (oldParent === newParent || nodeIndex === '__root') {
    return;
  }

  return _objectSpread(_objectSpread({}, nodes), {}, {
    [oldParent]: _objectSpread(_objectSpread({}, nodes[oldParent]), {}, {
      children: nodes[oldParent].children.filter(child => child !== nodeIndex)
    }),
    [newParent]: _objectSpread(_objectSpread({}, nodes[newParent]), {}, {
      children: [...nodes[newParent].children, nodeIndex]
    })
  });
}

function insertAtIndex(array, element, index) {
  return [...array.slice(0, index), element, ...array.slice(index, array.length)];
}

function moveNodeInChildren(nodes, nodeId, parent, index) {
  if (nodes[parent].children.indexOf(nodeId) === -1) {
    return;
  }

  return _objectSpread(_objectSpread({}, nodes), {}, {
    [parent]: _objectSpread(_objectSpread({}, nodes[parent]), {}, {
      children: insertAtIndex(nodes[parent].children.filter(childId => childId !== nodeId), nodeId, index)
    })
  });
}

function updateNodeDataProp(nodes, nodeId, name, prop) {
  return _objectSpread(_objectSpread({}, nodes), {}, {
    [nodeId]: _objectSpread(_objectSpread({}, nodes[nodeId]), {}, {
      data: _objectSpread(_objectSpread({}, nodes[nodeId].data), {}, {
        [prop]: name
      })
    })
  });
}

function removeNodeDataProp(nodes, nodeId, prop) {
  return _objectSpread(_objectSpread({}, nodes), {}, {
    [nodeId]: _objectSpread(_objectSpread({}, nodes[nodeId]), {}, {
      data: Object.keys(nodes[nodeId].data).reduce((newData, dataProp) => {
        if (prop !== dataProp) {
          newData[dataProp] = nodes[nodeId].data[dataProp];
        }

        return newData;
      }, {})
    })
  });
}

function updateNodeProp(nodes, nodeId, prop, value) {
  console.log(nodeId, prop, value);
  return _objectSpread(_objectSpread({}, nodes), {}, {
    [nodeId]: _objectSpread(_objectSpread({}, nodes[nodeId]), {}, {
      props: _objectSpread(_objectSpread({}, nodes[nodeId].props), {}, {
        [prop]: value
      })
    })
  });
}